module.exports = function(grunt) {

	var _ = require('lodash');
	var util = {};
	var Handlebars = require('handlebars');
	var fs = require("fs");
	var jsdom = require('jsdom');
	var cheerio = require('cheerio');


	util.strEndsWith = function(str, suffix) {
		return str.indexOf(suffix, str.length - suffix.length) !== -1;
	};

	util.removeXmlInstruction = function(xmlTxt) {
		var position = xmlTxt.indexOf('\n');
		if (position != -1) {
			if (xmlTxt.indexOf("<?xml") != -1) {
				xmlTxt = xmlTxt.substr(position + 1);
			}
		}
		return xmlTxt;
	}

	// options parser: (parse5,htmlparser2)
	// handlebars:true
	// remove_xml_instruction:true
	util.jsdomParseSync = function(txt, options) {
		if (options === undefined) {
			options = {
				'handlebars' : {},
				'remove_xml_instruction' : true
			};
		}

		var rxi = options['remove_xml_instruction'] === undefined ? true : options['remove_xml_instruction'];
		if (rxi) {
			txt = util.removeXmlInstruction(txt);
		}

		if (options['handlebars']) {
			var context = options['handlebars']['context'] ? options['handlebars']['context'] : {};
			txt = Handlebars.compile(txt)(context);
		}

		var jsdom_opts = [];
		if (options['parser'] = undefined) {
			var parser_name = options['parser'];
			var parser = require(parser_name);
			jsdom_opts['parser'] = parser;
		}
		var doc = jsdom.jsdom(txt, jsdom_opts);
		return doc;
	}








	util.footnoteHelper = function(element, notes_key,class_name, id_prefix, sup_prefix, options){

		//var Handlebars  = require('handlebars');

		var data = options['data'];
		var context = data[element] ? data[element] : {};

		var notes = context[notes_key] ? context[notes_key] : {} ;
		//console.log(context);

		var cheerio = require('cheerio');
		var $=  cheerio.load('<ol class="annotations">',{
		    	normalizeWhitespace: false,
		    	xmlMode: true,
		    	decodeEntities: false
				});
		var root = $('ol');



		for (var id in notes){
			var note = notes[id];
			var li = $('<li>');
			li.attr('id','bottom-' + id_prefix +id);
			var span = $('<span class="note-back">');
			var a = $('<a class="note-backlink">');
			a.html('^');
			a.attr('href','#ref-'+ id_prefix +id);
			span.append(a);
			li.append(span);
			var span = $('<span class="snote">');
			span.html(note);
			li.append(span);
			root.append(li);

		}

		//console.log($.html());
		return new Handlebars.SafeString($.html());
	}


	//element : note
	//notes_key: notes
	//class_name:anote
	//id_prefix:note-
	//sup_prefix: 'n '
	util.noteHelper = function(element,notes_key,class_name, id_prefix,sup_prefix, id, options) {

		//console.log('noteHelper',element,notes_key,class_name,sup_prefix,id);
		//var Handlebars = require('handlebars');
		var body = options.fn(this).trim();

		var data = options['data'];

		var context = data[element];
		var n = 1;
		var notes_n_key = notes_key+'-n';
		if (context === undefined) {
			context = {
			};
			context[notes_n_key] = 1;
			context[notes_key] = {};
		} else {
			n = context[notes_n_key];
			n = n + 1;
			context[notes_n_key] = n;
		}
		data[element] = context;
		var notes = data['root'][notes_key];
		var note = notes[id];
		if (note === undefined) {
			note = ''+id;
		}
		note = note.trim();
		var handlebars_context = data['root'];
		note = Handlebars.compile(note)(handlebars_context);
		context[notes_key][n] = note;
		//sconsole.log("CONTEXT",notes_key,context[notes_key]);

		var $ = cheerio.load('<a class="' + class_name + '">', {
			normalizeWhitespace : false,
			xmlMode : true,
			decodeEntities : false
		});

		var root = $('a.' + class_name);
		var nspan_id = id_prefix + n;
		root.attr('href','#bottom-'+nspan_id);
		var sup_id = 'ref-' + nspan_id;
		root.attr('id',sup_id);
		//var sup = '<sup id="'  + sup_id  + '" class="'  + class_name +  '">[' + sup_prefix + n + ']</sup>';
		var sup = '<sup  class="'  + class_name +  '">[' + sup_prefix + n + ']</sup>';
		var reftxt = body + sup;
		root.html(reftxt);

		var nspan = $('<span>');
		nspan.attr('id', nspan_id);
		nspan.attr('class',  class_name);
		nspan.html(note);

		root.after(nspan);


		//console.log($.html());
		return new Handlebars.SafeString($.html());

	}

	
	
	util.postImport= function(data){

		if (data['notes'] !== undefined){
			var notes={};
			var opts={'handlebars':{'context':data}};
			var doc = autil.jsdomParseSync(data['notes'],opts);
			var window = doc.parentWindow;
			var $ = jquery(window);
			$('note').each(function(idx,el){
				var el = $(el);
				var id = el.attr('id');
				var note_val = $(el).html();
				notes[id] = note_val;
			});
			data['notes'] = notes;
		}

		if (data['citations'] !== undefined){

			var citations={};
			var opts={'handlebars':{'context':data}};
			var doc = autil.jsdomParseSync(data['citations'],opts);
			var window = doc.parentWindow;
			var $ = jquery(window);
			$('citation').each(function(idx,el){
				var el = $(el);
				var id = el.attr('id');
				var c_val = $(el).html();
				citations[id] = c_val;
			});
			data['citations'] = citations;
		}

		if (data['references'] !== undefined){
			var references={};
			var opts={'handlebars':{'context':data}};
			var doc = autil.jsdomParseSync(data['references'],opts);
			var window = doc.parentWindow;
			var $ = jquery(window);
			$('reference').each(function(idx,el){
				var el = $(el);
				//var id = 'ref'+idx;
				var id = el.attr('id');
				var c_val = $(el).html();
				references[id] = c_val;
			});
			data['references'] = references;
		}

		if (data['dictionary'] !== undefined){
			var dictionary={};
			var opts={'handlebars':{'context':data}};
			var doc = autil.jsdomParseSync(data['dictionary'],opts);
			var window = doc.parentWindow;
			var $ = jquery(window);
			$('lemma').each(function(idx,el){
				var el = $(el);
				//var id = 'ref'+idx;
				var id = el.attr('id');
				//var c_val = $(el).html();
				var headword = el.find('headword').html();
				var definition = el.find('definition').html();
				dictionary[id] = {'headword':headword,'definition':definition};
			});
			data['dictionary'] = dictionary;
		}


	};

	
	
	
	
	
	
	return util;

};