

module.exports = function (grunt) {



	var fs = require("fs"),
	vm = require('vm'),
	jsesc = require('jsesc'),
	util = require('util'),
	Handlebars = require('handlebars'),
	process = require('process'),
	path = require('path'),
	xmldoc = require('xmldoc')
	_ = require('lodash'),
	jquery = require('jquery'),
	autil = require('./al-util.js')(grunt);


	var options = grunt.file.readJSON('etc/options.json');
	if( grunt.file.exists('options.json') ){
		options =grunt.file.readJSON('options.json');
	}

	var storage_file = options['storage'] ?  options['storage'] : 'etc/storage.json';
	var pr = { "src": "doc/examples/R" };
	if( grunt.file.exists(storage_file) ){
		pr = grunt.file.readJSON(storage_file);
	}

	var dataContext = {
		fs : fs,
		console : console,
		jsesc : jsesc,
		util : util,
		Handlebars : Handlebars,
		grunt : grunt,
		process : process,
		path : path,
		jquery:jquery,
		_ : _,
		autil : autil,
	};

	var adata = {};
	var article_data_file =  pr.src + '/data/article.js';
	if (grunt.file.exists(article_data_file)){
		vm.runInNewContext(fs.readFileSync(article_data_file), dataContext);
		adata = dataContext['adata'];
	}

	vm.runInNewContext(fs.readFileSync('etc/shell-env.js'), dataContext);
	var shell_env = dataContext['env'];
	adata['env'] = shell_env;

	adata['storage'] = pr;
	adata['options'] = options;
	//var xmldoc = require('xmldoc');
//	var strEndsWith = function(str, suffix) {
//		return str.indexOf(suffix, str.length - suffix.length) !== -1;
//	};


		var loadData = function(path,fname,vname){
			var data = null;
			var name = vname;
			if (autil.strEndsWith(fname, '.js')) {
				name = name == null ?  fname.substring(0, fname.length - 3) : name;
				vm.runInNewContext(fs.readFileSync(path), dataContext);
				data = dataContext[name];
			}else if (autil.strEndsWith(fname, '.xml')) {
				name = name == null ?  fname.substring(0, fname.length - 4) : name;
				data = grunt.file.read(path);
				//data = new xmldoc.XmlDocument(fs.readFileSync(path));
			} else if (autil.strEndsWith(fname, '.json')) {
				name = name == null ?  fname.substring(0, fname.length - 5) : name;
				data = grunt.file.readJSON(path);
			} else if (autil.strEndsWith(fname, '.txt')) {
				name = name == null ?  fname.substring(0, fname.length - 4) : name;
				data = grunt.file.read(path);
			}

			if (data != null && name != null){
				if(adata[name]!== undefined){
					adata[name + '-org'] = adata[name];
					data = _.merge(adata[name],data);
				}
				adata[name] = data;
			}
		};

	var imports = adata['import'];
	for ( var i in imports) {
		var fname = imports[i];
		var vname = null;
		if (_.isPlainObject(fname)){
			if (fname['file'] === undefined){
				console.log(fname);
				throw Error("ERROR import-file property 'file' missing ");
			} else if (fname['variable'] === undefined){
					console.log(fname);
					throw Error("ERROR import-file property 'variable' missing ");
			} else {
				vname = fname['variable'];
				fname = fname['file'];
			}
		}

		var path = pr.src + '/data/' + fname;
		if (fs.existsSync(path)) {
			loadData(path, fname,vname);
		} else {
			throw Error("ERROR loading data canot find " +  path);
		}
	}

	var data_generated_dir =  pr.src + '/data/generated/';
	if (grunt.file.exists(data_generated_dir) && grunt.file.isDir(data_generated_dir)){
		grunt.file.recurse(data_generated_dir, function(abspath, rootdir, subdir, filename){
			var fname = filename;
			if (! autil.strEndsWith(fname, '.bak') && ! autil.strEndsWith(fname, '.org')) {
				loadData(abspath, filename);
			}
		});
	}


	var helpers = [];
	grunt.file.recurse('build/helpers/', function(abspath, rootdir, subdir, filename){
		var fname = filename;
		if (autil.strEndsWith(filename, '.js')) {
			name = filename.substring(0, filename.length - 3);
			 Handlebars.registerHelper(name, require(fs.realpathSync(abspath)));
			 helpers.push(name);
		}
	});


	autil.postImport(adata);

	var path=pr.src + '/data/postImport.js';
	if (fs.existsSync(path)) {
		vm.runInNewContext(fs.readFileSync(path), dataContext);
		var postImport = dataContext['postImport'];
		postImport(adata);
	}


	_.each(helpers,function(name,idx){
		Handlebars.unregisterHelper(name);
	});

	return adata;

};
