#
# article charts
#

library(rjson);
library(plotrix);
library(Hmisc);
set.seed(3200);


img_width = 3.1;
img_height = 3.0;


# params
####################
#m=180; #mean
#sd=8;  #standar deviation
#n=800; #number of samples


json_data_input <- fromJSON(file="in/article.json");

m=json_data_input[['R-in']]$mean;
sd=json_data_input[['R-in']]$sd;
n=json_data_input[['R-in']]$size;
imgdata = json_data_input$images
jdata = list();

t1  = function(n,p,img_width,img_height){
  x=seq(0,n,by=1);
  y=dbinom(x, size=n, prob=p);
  title = paste(n,' coin rolls');
  #title = paste(title,' (t1)');
  svg(paste(n, "coins.svg",sep=""),width=img_width,height=img_height);
  barp(y,names.arg=x,main=title,
       ylab="probability",
       xlab="number of heads",
       col = "lightgrey",do.first=grid());
  #grid();
  dev.off();

}

t1(2,0.5,img_width,img_height);
t1(3,0.5,img_width,img_height);
t1(8,0.5,img_width,img_height);
t1(15,0.5,img_width,img_height);
t1(16,0.5,img_width,img_height);
t1(32,0.5,img_width,img_height);
t1(64,0.5,img_width,img_height);




t2  = function(n,p,img_width,img_height){
  x=seq(0,n,by=1);
  y=dbinom(x, size=n, prob=p);
  title = paste(n,' coin rolls');
  #title = paste(title,' (t2)');
  m=n*p; #mesos
  x=x-m;
	svg(paste(n, "coins-f1.svg",sep=""),width = img_width,height=img_height);
  barp(y,names.arg=x,main=title,ylab="probability",xlab="diff from mean",col = "lightgrey",do.first=grid());
  #grid();
  dev.off();


}


t2(64,0.5,img_width,img_height);


t3  = function(n,p,img_width,img_height){
  x=seq(0,n,by=1);
  y=dbinom(x, size=n, prob=p);
  title = paste(n,' coin rolls');
  #title = paste(title,' (t3)');
  m=n*p; #mesos
  d=sqrt(n*p*(1-p)); #tipiki apoklisi
  x=x-m;
  x=x/d;
  y=y*d;
  #x=round(x,1);
  #y=round(y,1);
  df=subset(data.frame(x,y), abs(x) <3.2);
  #df=subset(data.frame(x,y), y>0);
  #barp(df$y,names.arg=df$x,main=title,ylab="y",xlab="x",col = "lightgrey");
  #grid();


  x=df$x;
  y=df$y;
  xrange <- range(x);# + c(-0.1, +0.1)
  yrange <- c(0, max(y));

  svg(paste(n, "coins-f2.svg",sep=""),width = img_width,height=img_height);
  plot(xrange, yrange, type = "n", xlab = "x", ylab = "y", xaxt = "n",main=title)
  # draw bars using boxplot and density using curve
  boxplot(as.data.frame(rbind(0,y)), at = x, names = x,
          boxwex = 1/(d*1.04), medlty = 0, add = TRUE, col = "lightgrey");
  dev.off();

}


t3(64,0.5,img_width,img_height);







t5 = function(n,p,img_width,img_height){
  q=1-p;
  m=n*p; #mesos
  d=sqrt(n*p*q); #tipiki apoklisi
  x=seq(0,n,by=1);
  y=dbinom(x, size=n, prob=p);
  x=x-m;
  x1=x;
  x=x/d;
  y=y*d;

  df=subset(data.frame(x,y), abs(x) <3.2);
  x=df$x;
  y=df$y;


  y2=dnorm(x, m=0, sd=1);

  title = paste(n,'coins mean: ',round(m,2),' std: ',round(d,2));
  #plot(x,y,main=title,ylab='y',xlab='x');grid();
  #lines(x,y2,col="green");

  #y=y2;
  # setup plot ranges and axes
  xrange <- range(x);# + c(-0.1, +0.1)
  yrange <- c(0, max(y))

  svg(paste(n, "coins-f3.svg",sep=""),width = img_width,height=img_height);
  plot(xrange, yrange, type = "n", xlab = "x", ylab = "y", xaxt = "n",main=title)
  # draw bars using boxplot and density using curve
  boxplot(as.data.frame(rbind(0,y)), at = x, names = x,
          boxwex = 1/(d*1.04), medlty = 0, add = TRUE, col = "lightgrey");
  curve(dnorm(x, m=0, sd=1), min(xrange), max(xrange), add = TRUE);
  #grid();
  dev.off();



};


t5(64,0.5,img_width,img_height);







t6 = function(n,p,img_width,img_height){
	q=1-p;
	m=n*p; #mesos
	sd=sqrt(n*p*q); #tipiki apoklisi
	x=seq(0,n,by=1);
	y=dbinom(x, size=n, prob=p);

	df=subset(data.frame(x,y), abs(x-m) <(3.2*sd));
	x=df$x;
	y=df$y;

	y2=dnorm(x, m=m, sd=sd);

	title = paste(n,'coins mean: ',round(m,2),' std: ',round(sd,2));
	#plot(x,y,main=title,ylab='y',xlab='x');grid();
	#lines(x,y2,col="green");

	# setup plot ranges and axes
	xrange <- range(x);# + c(-0.1, +0.1)
	yrange <- c(0, max(y))

	svg(paste(n, "coins-normal.svg",sep=""),width = img_width,height=img_height);
	plot(xrange, yrange, type = "n", xlab = "x", ylab = "y", xaxt = "n",main=title)
	# draw bars using boxplot and density using curve
	boxplot(as.data.frame(rbind(0,y)), at = x, names = x,
			boxwex = 0.96, medlty = 0, add = TRUE, col = "lightgrey");
	curve(dnorm(x, m=m, sd=sd), min(xrange), max(xrange), add = TRUE);
	#grid();
	dev.off();



};


t6(64,0.5,img_width,img_height);














set.seed(3200);

#img_width = 340;
#img_height = 330;



h <- sort(round(rnorm(n=n, m=m, sd=sd), digits = 0));
summary(h);
describe(h);

freqs = as.data.frame(table(h));
m=mean(h);
sd=sd(h);
#cat("population:",n,"persons (samples)","\n");
#cat("mean:",m,"\n");
#cat("std:",sd,"\n");


jdata['size'] = n;
jdata['mean'] = round(m,1);
jdata['sd'] = round(sd,1);
jdata['median'] = median(h);
jdata['unique'] = length(unique(h));

hmin = min(h);
hmax = max(h);
jdata['min'] = hmin;
jdata['max'] = hmax;


svg("normal-population.svg",width = img_width,height=img_height);
plot(h,ylab="height",main=paste("heights of",n,"persons"));
grid();
dev.off();


svg("normal-hist1.svg",width = img_width,height=img_height);
par(mar=c(4.2,4.2,3,1)); #BLTR
histbreaks = ceiling((hmax-hmin)/2);
cat("bins1:",histbreaks,"\n");
jdata['bins1'] = histbreaks;
hist(h, prob=TRUE, main="",breaks=histbreaks, xlab="",ylab="");
grid();
hist(h, prob=TRUE, main="",breaks=histbreaks, xlab="",ylab="",add = TRUE, col="white");
title(main="histogram", cex.main = 0.8);
title(xlab="height", cex.xlab= 0.8);
title(ylab="Density", cex.xlab= 0.8);
dev.off();

svg("normal-hist2.svg",width = img_width,height=img_height);
par(mar=c(4.2,4.2,3,1)); #BLTR
histbreaks = ceiling((hmax-hmin)/8);
cat("bins2:",histbreaks,"\n");
jdata['bins2'] = histbreaks;
hist(h, prob=TRUE, main="",breaks=histbreaks, xlab="",ylab="");
grid();
hist(h, prob=TRUE, main="",breaks=histbreaks, xlab="",ylab="",add = TRUE, col="white");
title(main="histogram", cex.main = 0.8);
title(xlab="height", cex.xlab= 0.8);
title(ylab="Density", cex.xlab= 0.8);


dev.off();


x=as.numeric(levels(freqs$h));
y=freqs$Freq;
svg("normal-barp1.svg",width = img_width,height=img_height);
par(mar=c(4.2,4.2,3,1)); #BLTR
barp(y,names.arg=x,main="", ylab="", xlab="", col = "lightgrey",do.first=grid());
title(main="barplot", cex.main = 0.8);
title(xlab="height", cex.xlab= 0.8);
title(ylab="Freq", cex.xlab= 0.8);
#grid();
dev.off();



xp=y/n;
sum(xp);
svg("normal-barp2.svg",width = img_width,height=img_height);
par(mar=c(4.2,4.2,3,1)); #BLTR
barp(xp,names.arg=x,
		main="",
		ylab="",
		xlab="",
		col = "lightgrey",do.first=grid());
title(main="barplot", cex.main = 0.8);
title(xlab="height", cex.xlab= 0.8);
title(ylab="Prob", cex.xlab= 0.8);

#grid();
dev.off();


probs = data.frame(x,xp);
p1=subset(probs,x==180)$xp;
cat("prob180",p1,"\n");
p1r = round(p1,2);
jdata['prob180'] = p1r;
jdata['prob180p'] = paste(p1r * 100,"%",sep="");


cat('@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@\n');
svg("normal.svg",width = img_width,height=img_height);
histbreaks = ceiling((hmax-hmin+1));
#cat("bins3:",histbreaks,"\n");
jdata['bins3'] = histbreaks;
par(mar=c(4.2,4.2,3,1)); #BLTR
hist(h, prob=TRUE, ylim=c(0,.06), breaks=histbreaks,xlab="", main="",ylab="");
grid();
#abline(v=(seq(160,200,by=10)), col="lightgray", lty="dotted");
#abline(h=(seq(0,0.06,by=0.01)), col="lightgray", lty="dotted");
hist(h, prob=TRUE, add = TRUE, col="white", ylim=c(0,0.06), breaks=histbreaks,xlab="", main="");
title(main="PDF", cex.main = 0.8);
title(xlab="heigh", cex.xlab= 0.8);
title(ylab="Density", cex.xlab= 0.8);
print(par());
curve(dnorm(x, m, sd), add=TRUE, col="red", lwd=3);
xmin=175;
xmax=185;
xmax2= xmax+1;
densities=dnorm(x,mean=m,sd=sd);
x2=seq(xmin,xmax2,length=200);
y2=dnorm(x2,mean=m,sd=sd);
polygon(c(xmin,x2,xmax2),c(0,y2,0),col="pink",density=50);
dev.off();

cat('@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@\n');

pt1 = 0;
for (i in xmin:(xmax2-1)){
	#cat("i=",i,"\n");
	tmp = subset(probs,x==i)$xp;
	if (length(tmp)>0){
		pt1 = pt1+tmp;
		#cat("pt1=",pt1,"\n");
	}
}

pt2= pnorm(xmax2,mean=m,sd=sd)-pnorm(xmin,mean=m,sd=sd);
#pt1;
#pt2;
diff = abs(pt1-pt2);
cat("---------------------------\n");
cat("range",xmin,"-",xmax,"\n");
cat("real probability:",pt1,"\n");
cat("computed from normal:",pt2,"\n");
cat("diff:",diff,"\n");

jdata['probavg']= round(pt1,2);
jdata['probavgp']= paste(round(pt1,2)*100,'%',sep='');
jdata['probavgreal']= round(pt1,4);

jdata['probavnormal'] = round(pt2,4);
jdata['probavgdiff'] = round(diff,4);



cat("========================================\n");
cat("JSON DATA\n");
cat("========================================\n");
cat(toJSON(jdata));



#jout=list(R=list(sample = jdata));
jout=list(sample = jdata);

zz <- file("R.json", "w")  # open an output file connection
cat(toJSON(jout), file = zz, sep = "\n")
close(zz)



